From: Jonathan Brassow <jbrassow@redhat.com>

Let the exception store types print out their status through
the new API, rather than having the snapshot code do it.

Adjust the buffer position to allow for the preceding DMEMIT in the
arguments to type->status().

Signed-off-by: Jonathan Brassow <jbrassow@redhat.com>
Signed-off-by: Alasdair G Kergon <agk@redhat.com>

---
 drivers/md/dm-exception-store.h |    5 +++--
 drivers/md/dm-snap-persistent.c |   16 ++++++++++++----
 drivers/md/dm-snap-transient.c  |   16 ++++++++++++----
 drivers/md/dm-snap.c            |    8 ++------
 4 files changed, 29 insertions(+), 16 deletions(-)

Index: linux-2.6.29/drivers/md/dm-exception-store.h
===================================================================
--- linux-2.6.29.orig/drivers/md/dm-exception-store.h	2009-04-02 16:49:51.000000000 +0100
+++ linux-2.6.29/drivers/md/dm-exception-store.h	2009-04-02 16:49:52.000000000 +0100
@@ -79,8 +79,9 @@ struct dm_exception_store_type {
 	 */
 	void (*drop_snapshot) (struct dm_exception_store *store);
 
-	int (*status) (struct dm_exception_store *store, status_type_t status,
-		       char *result, unsigned int maxlen);
+	unsigned (*status) (struct dm_exception_store *store,
+			    status_type_t status, char *result,
+			    unsigned maxlen);
 
 	/*
 	 * Return how full the snapshot is.
Index: linux-2.6.29/drivers/md/dm-snap-persistent.c
===================================================================
--- linux-2.6.29.orig/drivers/md/dm-snap-persistent.c	2009-04-02 16:49:46.000000000 +0100
+++ linux-2.6.29/drivers/md/dm-snap-persistent.c	2009-04-02 16:49:52.000000000 +0100
@@ -688,11 +688,19 @@ static int persistent_ctr(struct dm_exce
 	return 0;
 }
 
-static int persistent_status(struct dm_exception_store *store,
-			     status_type_t status, char *result,
-			     unsigned int maxlen)
+static unsigned persistent_status(struct dm_exception_store *store,
+				  status_type_t status, char *result,
+				  unsigned maxlen)
 {
-	int sz = 0;
+	unsigned sz = 0;
+
+	switch (status) {
+	case STATUSTYPE_INFO:
+		break;
+	case STATUSTYPE_TABLE:
+		DMEMIT(" %s P %llu", store->cow->name,
+		       (unsigned long long)store->chunk_size);
+	}
 
 	return sz;
 }
Index: linux-2.6.29/drivers/md/dm-snap-transient.c
===================================================================
--- linux-2.6.29.orig/drivers/md/dm-snap-transient.c	2009-04-02 16:49:46.000000000 +0100
+++ linux-2.6.29/drivers/md/dm-snap-transient.c	2009-04-02 16:49:52.000000000 +0100
@@ -81,11 +81,19 @@ static int transient_ctr(struct dm_excep
 	return 0;
 }
 
-static int transient_status(struct dm_exception_store *store,
-			    status_type_t status, char *result,
-			    unsigned maxlen)
+static unsigned transient_status(struct dm_exception_store *store,
+				 status_type_t status, char *result,
+				 unsigned maxlen)
 {
-	int sz = 0;
+	unsigned sz = 0;
+
+	switch (status) {
+	case STATUSTYPE_INFO:
+		break;
+	case STATUSTYPE_TABLE:
+		DMEMIT(" %s N %llu", store->cow->name,
+		       (unsigned long long)store->chunk_size);
+	}
 
 	return sz;
 }
Index: linux-2.6.29/drivers/md/dm-snap.c
===================================================================
--- linux-2.6.29.orig/drivers/md/dm-snap.c	2009-04-02 16:49:51.000000000 +0100
+++ linux-2.6.29/drivers/md/dm-snap.c	2009-04-02 16:49:52.000000000 +0100
@@ -70,9 +70,6 @@ struct dm_snapshot {
 	/* Origin writes don't trigger exceptions until this is set */
 	int active;
 
-	/* Used for display of table */
-	char type;
-
 	mempool_t *pending_pool;
 
 	atomic_t pending_exceptions_count;
@@ -1166,9 +1163,8 @@ static int snapshot_status(struct dm_tar
 		 * make sense.
 		 */
 		DMEMIT("%s", snap->origin->name);
-		DMEMIT(" %s %s %llu", snap->store->cow->name,
-		       snap->store->type->name,
-		       (unsigned long long)snap->store->chunk_size);
+		snap->store->type->status(snap->store, type, result + sz,
+					  maxlen - sz);
 		break;
 	}
 
